"""Resource modules for RDF-Jena."""

from . import owl_vocabularies, prefix_mappings, query_templates, shacl_shapes

__all__ = [
    "owl_vocabularies",
    "prefix_mappings",
    "query_templates",
    "shacl_shapes",
]
